@@ -1,9 +1,11 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE TemplateHaskell #-}
 
--- to be processed using hsb2hs
-module Text.CSL.Data.Embedded (localeFiles, defaultCSL, manpage, license) where
-import Data.FileEmbed
+module Text.CSL.Data.Embedded (localeFiles, defaultCSL, manpage, license)
+where
+import Prelude
 import qualified Data.ByteString.Char8 as S
+import           Data.FileEmbed
 
 localeFiles :: [(FilePath, S.ByteString)]
 localeFiles = $(embedDir "locales")
