@@ -1,6 +1,7 @@
 {-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE FlexibleContexts    #-}
 {-# LANGUAGE PatternGuards       #-}
+{-# LANGUAGE LambdaCase          #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 -----------------------------------------------------------------------------
 -- |
@@ -32,7 +33,7 @@ import           Data.Maybe
 import           Data.Monoid            (Any (..))
 import           Data.String            (fromString)
 import qualified Data.Text              as T
-import           Text.Pandoc.Definition (Inline (Link, Str), nullAttr)
+import           Text.Pandoc.Definition (Inline (Link, Span, Str), nullAttr)
 import           Text.Pandoc.Shared     (stringify)
 import           Text.Pandoc.Walk       (walk)
 
@@ -44,7 +45,7 @@ import           Text.CSL.Exception
 import           Text.CSL.Output.Plain
 import           Text.CSL.Reference
 import           Text.CSL.Style         hiding (Any)
-import           Text.CSL.Util          (betterThan, isRange, last', proc,
+import           Text.CSL.Util          (orIfNull, isRange, last', proc,
                                          proc', query, readNum, safeRead)
 
 -- | Produce the output with a 'Layout', the 'EvalMode', a 'Bool'
@@ -130,7 +131,8 @@ evalElement el
     | Group        fm d l   <- el = outputList fm d <$> tryGroup l
     | Date{} <- el = evalDate el
     | Label    s f fm _     <- el = formatLabel f fm True s -- FIXME !!
-    | Term     s f fm p     <- el = formatTerm  f fm p    s
+    | Term     s f fm p     <- el = getStringVar "ref-id" >>= \refid ->
+                                      formatTerm  f fm p refid  s
     | Names    s n fm d sub <- el = modify (\st -> st { contNum = [] }) >>
                                     ifEmpty (evalNames False s n d)
                                             (withNames s el $ evalElements sub)
@@ -159,12 +161,15 @@ evalElement el
     where
       addSpaces strng = (if take 1 strng == " " then (OSpace:) else id) .
                         (if last' strng == " " then (++[OSpace]) else id)
-      substituteWith e = head <$> gets (names . env) >>= \(Names _ ns fm d _) ->
-                           case e of
-                             Names rs [Name NotSet fm'' [] [] []] fm' d' []
-                                 -> let nfm = mergeFM fm'' $ mergeFM fm' fm in
-                                    evalElement $ Names rs ns nfm (d' `betterThan` d) []
-                             _   -> evalElement e
+      substituteWith e =
+        gets (names . env) >>= \case
+          (Names _ ns fm d _ : _) -> evalElement $ proc replaceNames e
+             where
+               replaceNames (Names rs [Name NotSet fm'' [] [] []] fm' d' []) =
+                  let nfm = mergeFM fm'' $ mergeFM fm' fm in
+                  Names rs ns nfm (d' `orIfNull` d) []
+               replaceNames x = x
+          _ -> return []
 
       -- from citeproc documentation: "cs:group implicitly acts as a
       -- conditional: cs:group and its child elements are suppressed if
@@ -219,6 +224,10 @@ evalElement el
              consumeVariable s >> formatTitle s f fm
         | otherwise =
              case map toLower s of
+               "first-reference-note-number"
+                             -> do refid <- getStringVar "ref-id"
+                                   return [Output [OPan [Span ("",["first-reference-note-number"],[("refid",refid)]) [Str "0"]]] fm]
+
                "year-suffix" -> getStringVar "ref-id" >>= \k  ->
                                 return . return $ OYearSuf [] k [] fm
                "page"        -> getStringVar "page" >>= formatRange fm
@@ -241,11 +250,11 @@ evalElement el
                "pmid"        -> getStringVar "pmid" >>= \d ->
                                 if null d
                                    then return []
-                                   else return [Output [OPan [Link nullAttr [Str d] ("http://www.ncbi.nlm.nih.gov/pubmed/" ++ d, "")]] fm]
+                                   else return [Output [OPan [Link nullAttr [Str d] ("https://www.ncbi.nlm.nih.gov/pubmed/" ++ d, "")]] fm]
                "pmcid"       -> getStringVar "pmcid" >>= \d ->
                                 if null d
                                    then return []
-                                   else return [Output [OPan [Link nullAttr [Str d] ("http://www.ncbi.nlm.nih.gov/pmc/articles/" ++ d, "")]] fm]
+                                   else return [Output [OPan [Link nullAttr [Str d] ("https://www.ncbi.nlm.nih.gov/pmc/articles/" ++ d, "")]] fm]
                _ -> do (opts, as) <- gets (env >>> options &&& abbrevs)
                        r <- getVar []
                               (getFormattedValue opts as f fm s) s
@@ -305,8 +314,8 @@ getFormattedValue o as f fm s val
                in  if null ys
                       then []
                       else [Output [OPan $ walk value' ys] fm]
-    | Just v <- fromValue val :: Maybe String    = (:[]) . flip OStr fm . Data.Maybe.fromMaybe v . getAbbr $ value v
-    | Just v <- fromValue val :: Maybe Literal   = (:[]) . flip OStr fm . Data.Maybe.fromMaybe (unLiteral v) . getAbbr $ value $ unLiteral v
+    | Just v <- fromValue val :: Maybe String    = maybe [] (\x -> [OStr x fm]) . getAbbr $ value v
+    | Just v <- fromValue val :: Maybe Literal   = maybe [] (\x -> [OStr x fm]) . getAbbr $ value $ unLiteral v
     | Just v <- fromValue val :: Maybe Int       = output  fm (if v == 0 then [] else show v)
     | Just v <- fromValue val :: Maybe Int       = output  fm (if v == 0 then [] else show v)
     | Just v <- fromValue val :: Maybe CNum      = if v == 0 then [] else [OCitNum (unCNum v) fm]
@@ -366,7 +375,7 @@ formatNumber f fm v n
                     _           -> maybe "" show . (safeRead :: String -> Maybe Int)
 
       roman :: Int -> String
-      roman     = foldr (++) [] . reverse . map (uncurry (!!)) . zip romanList .
+      roman     = concat . reverse . zipWith (!!) romanList .
                   map (readNum . return) . take 4 .
                   reverse . show
       romanList = [[ "", "i", "ii", "iii", "iv", "v", "vi", "vii", "viii", "ix" ]
