@@ -54,22 +54,24 @@ evalDate (Date s f fm dl dp dp') = do
   case f of
     NoFormDate -> outputList fm dl .
                   concatMap (formatDate em k tm dp) <$> mapM getDateVar s
-    _          -> do Date _ _ lfm ldl ldp _ <- getDate f
-                     let go dps = return . outputList (updateFM fm lfm) (if ldl /= [] then ldl else dl) .
-                                  concatMap (formatDate em k tm dps)
-                         update l x@(DatePart a b c d) =
-                             case filter ((==) a . dpName) l of
-                               (DatePart _ b' c' d':_) -> DatePart a (updateS  b b')
-                                                                     (updateS  c c')
-                                                                     (updateFM d d')
-                               _                       -> x
-                         updateDP = map (update dp) ldp
-                         date     = mapM getDateVar s
-                     case dp' of
-                       "year-month" -> go (filter ((/=) "day"  . dpName) updateDP) =<< date
-                       "year"       -> go (filter ((==) "year" . dpName) updateDP) =<< date
-                       _            -> go                                updateDP  =<< date
-
+    _          -> do res <- getDate f
+                     case res of
+                       Date _ _ lfm ldl ldp _ -> do
+                         let go dps = return . outputList (updateFM fm lfm) (if ldl /= [] then ldl else dl) .
+                                      concatMap (formatDate em k tm dps)
+                             update l x@(DatePart a b c d) =
+                                 case filter ((==) a . dpName) l of
+                                   (DatePart _ b' c' d':_) -> DatePart a (updateS  b b')
+                                                                         (updateS  c c')
+                                                                         (updateFM d d')
+                                   _                       -> x
+                             updateDP = map (update dp) ldp
+                             date     = mapM getDateVar s
+                         case dp' of
+                           "year-month" -> go (filter ((/=) "day"  . dpName) updateDP) =<< date
+                           "year"       -> go (filter ((==) "year" . dpName) updateDP) =<< date
+                           _            -> go                                updateDP  =<< date
+                       _ -> return []
 evalDate _ = return []
 
 getDate :: DateForm -> State EvalState Element
@@ -173,19 +175,23 @@ formatDate em k tm dp date
 ordinal :: [CslTerm] -> String -> Int -> String
 ordinal ts v s
     | s < 10        = let a = termPlural (getWith1 (show s)) in
-                      if  a == [] then setOrd (term []) else show s ++ a
+                      if  null a then setOrd (term []) else show s ++ a
     | s < 100       = let a = termPlural (getWith2 (show s))
                           b = getWith1 [last (show s)] in
-                      if  a /= []
+                      if  not (null a)
                       then show s ++ a
-                      else if termPlural b == [] || (termMatch b /= [] && termMatch b /= "last-digit")
-                           then setOrd (term []) else setOrd b
+                      else if null (termPlural b) ||
+                              (not (null (termMatch b)) &&
+                               termMatch b /= "last-digit")
+                           then setOrd (term [])
+                           else setOrd b
     | otherwise     = let a = getWith2  last2
                           b = getWith1 [last (show s)] in
-                      if termPlural a /= [] && termMatch a /= "whole-number"
+                      if not (null (termPlural a)) &&
+                         termMatch a /= "whole-number"
                       then setOrd a
                       else if null (termPlural b) ||
-                              (termMatch b /= [] &&
+                              (not (null (termMatch b)) &&
                                termMatch b /= "last-digit")
                            then setOrd (term [])
                            else setOrd b
