@@ -19,7 +19,7 @@ module Text.CSL.Eval.Names where
 
 import Prelude
 import           Control.Monad.State
-import           Data.Char              (isLetter, isLower, isUpper)
+import           Data.Char              (isLower, isUpper)
 import           Data.List              (intersperse, nub)
 import           Data.List.Split        (wordsBy)
 import           Data.Maybe             (isJust)
@@ -278,9 +278,12 @@ formatName m b f fm ops np n
                      $ splitStrWhen (=='-') x
 
       sortSep g s = when_ g $ separator ++ addAffixes (g <+> s) "given" mempty
-      separator   = if null (getOptionVal "sort-separator" ops)
-                    then [OPan [Str ",", Space]]
-                    else [OPan $ B.toList $ B.text $ getOptionVal "sort-separator" ops]
+      separator   = if isByzantineFamily
+                       then
+                         if null (getOptionVal "sort-separator" ops)
+                            then [OPan [Str ",", Space]]
+                            else [OPan $ B.toList $ B.text $ getOptionVal "sort-separator" ops]
+                       else []
 
       suff      = if commaSuffix n && nameSuffix n /= mempty
                   then suffCom
@@ -302,9 +305,25 @@ formatName m b f fm ops np n
       dropping  = droppingPart n
       nondropping  = nonDroppingPart n
       -- see src/load.js ROMANESQUE_REGEX in citeproc-js:
-      isByzantine c = not (isLetter c) ||
-                      c <= '\x5FF' ||
-                      (c >= '\x1e00' && c <= '\x1fff')
+      -- /[-0-9a-zA-Z\u0e01-\u0e5b\u00c0-\u017f\u0370-\u03ff\u0400-\u052f\u0590-\u05d4\u05d6-\u05ff\u1f00-\u1fff\u0600-\u06ff\u200c\u200d\u200e\u0218\u0219\u021a\u021b\u202a-\u202e]/
+      isByzantine c = c == '-' ||
+                      (c >= '0' && c <= '9') ||
+                      (c >= 'a' && c <= 'z') ||
+                      (c >= 'A' && c <= 'Z') ||
+                      (c >= '\x0e01' && c <= '\x0e5b') ||
+                      (c >= '\x00c0' && c <= '\x017f') ||
+                      (c >= '\x0370' && c <= '\x03ff') ||
+                      (c >= '\x0400' && c <= '\x052f') ||
+                      (c >= '\x0590' && c <= '\x05d4') ||
+                      (c >= '\x05d6' && c <= '\x05ff') ||
+                      (c >= '\x1f00' && c <= '\x1fff') ||
+                      (c >= '\x0600' && c <= '\x06ff') ||
+                      (c >= '\x200c' && c <= '\x200e') ||
+                      (c >= '\x2018' && c <= '\x2019') ||
+                      (c >= '\x021a' && c <= '\x021b') ||
+                      (c >= '\x202a' && c <= '\x202e')
+
+      isByzantineFamily = any isByzantine (stringify family)
       shortName = oPan' (unFormatted $ nondropping <+> family) (form "family")
 
       longName g
@@ -320,7 +339,7 @@ formatName m b f fm ops np n
                                                           in oPan' (unFormatted fam) (form "family") <> sortSep g par <> suffCom
         | otherwise = let fam = addAffixes (dropping <+> nondropping <+> family) "family" suff
                           gvn = oPan' (unFormatted g) (form "given")
-                      in  if all isByzantine $ stringify $ unFormatted family
+                      in  if isByzantineFamily
                           then gvn <++> fam
                           else fam <> gvn
 
@@ -343,8 +362,9 @@ formatName m b f fm ops np n
                          | disWithGiven, isLong        -> [longName givenLong]
                          | otherwise                   -> []
 
-formatTerm :: Form -> Formatting -> Bool -> String -> State EvalState [Output]
-formatTerm f fm p s = do
+formatTerm :: Form -> Formatting -> Bool -> String -> String
+           -> State EvalState [Output]
+formatTerm f fm p refid s = do
   plural <- if s `elem` ["page", "volume", "issue"]
                then do
                  varset <- isVarSet s
@@ -353,7 +373,10 @@ formatTerm f fm p s = do
                     else return p
                else return p
   t <- getTerm plural f s
-  return $ oStr' t fm
+  return $
+     if s == "no date"
+        then [OYear t refid fm]
+        else oStr' t fm
 
 formatLabel :: Form -> Formatting -> Bool -> String -> State EvalState [Output]
 formatLabel f fm p s
