@@ -38,7 +38,7 @@ import           Text.CSL.Compat.Pandoc (readLaTeX)
 import           Text.CSL.Exception     (CiteprocException (ErrorReadingBib, ErrorReadingBibFile))
 import           Text.CSL.Parser        (parseLocale)
 import           Text.CSL.Reference
-import           Text.CSL.Style         (Agent (..), CslTerm (..),
+import           Text.CSL.Style         (Agent (..), emptyAgent, CslTerm (..),
                                          Formatted (..), Locale (..))
 import           Text.CSL.Util          (onBlocks, protectCase, safeRead,
                                          splitStrWhen, trim, unTitlecase)
@@ -193,7 +193,8 @@ inQuotes = do
                       ) (char '"')
 
 fieldName :: BibParser String
-fieldName = map toLower <$> many1 (letter <|> digit <|> oneOf "-_:+")
+fieldName =
+  resolveAlias . map toLower <$> many1 (letter <|> digit <|> oneOf "-_:+")
 
 isBibtexKeyChar :: Char -> Bool
 isBibtexKeyChar c = isAlphaNum c || c `elem` ".:;?!`'()/*@_+=-[]*&"
@@ -225,6 +226,11 @@ entField = do
   spaces
   return (k, concat vs)
 
+resolveAlias :: String -> String
+resolveAlias "archiveprefix" = "eprinttype"
+resolveAlias "primaryclass" = "eprintclass"
+resolveAlias s = s
+
 rawWord :: BibParser String
 rawWord = many1 alphaNum
 
@@ -1003,7 +1009,7 @@ toLiteralList _ = mzero
 
 toAuthorList :: Options -> [Block] -> Bib [Agent]
 toAuthorList opts [Para xs] =
-  mapM (toAuthor opts) $ splitByAnd xs
+  filter (/= emptyAgent) <$> mapM (toAuthor opts) (splitByAnd xs)
 toAuthorList opts [Plain xs] = toAuthorList opts [Para xs]
 toAuthorList _ _ = mzero
 
@@ -1030,6 +1036,25 @@ toAuthor _ [Span ("",[],[]) ils] =
           , commaSuffix     = False
           , parseNames      = False
           }
+ -- extended BibLaTeX name format - see #266
+toAuthor _ ils@(Str ys:_) | '=' `elem` ys = do
+  let commaParts = splitWhen (== Str ",")
+                   $ splitStrWhen (\c -> c == ',' || c == '=' || c == '\160')
+                   $ ils
+  let addPart ag (Str "given" : Str "=" : xs) =
+        ag{ givenName = givenName ag ++ [Formatted xs] }
+      addPart ag (Str "family" : Str "=" : xs) =
+        ag{ familyName = Formatted xs }
+      addPart ag (Str "prefix" : Str "=" : xs) =
+        ag{ droppingPart = Formatted xs }
+      addPart ag (Str "useprefix" : Str "=" : Str "true" : _) =
+        ag{ nonDroppingPart = droppingPart ag, droppingPart = mempty }
+      addPart ag (Str "suffix" : Str "=" : xs) =
+        ag{ nameSuffix = Formatted xs }
+      addPart ag (Space : xs) = addPart ag xs
+      addPart ag _ = ag
+  return $ foldl' addPart emptyAgent commaParts
+
 -- First von Last
 -- von Last, First
 -- von Last, Jr ,First
@@ -1221,6 +1246,8 @@ itemToReference lang locale bibtex caseTransform = bib $ do
                                          Lang "en" _ -> caseTransform
                                          _           -> False }
   id' <- asks identifier
+  otherIds <- (map trim . splitWhen (==',') <$> getRawField "ids")
+                <|> return []
   et <- asks entryType
   guard $ et /= "xdata"
   opts <- (parseOptions <$> getRawField "options") <|> return []
@@ -1460,15 +1487,21 @@ itemToReference lang locale bibtex caseTransform = bib $ do
 
   -- url, doi, isbn, etc.:
   -- note that with eprinttype = arxiv, we take eprint to be a partial url
+  -- archivePrefix is an alias for eprinttype
   url' <- (guard (et == "online" || lookup "url" opts /= Just "false")
            >> getRawField "url")
        <|> (do etype <- getRawField "eprinttype"
                eprint <- getRawField "eprint"
-               case map toLower etype of
-                    "arxiv"       -> return $ "http://arxiv.org/abs/" ++ eprint
-                    "googlebooks" -> return $ "http://books.google.com?id=" ++
-                                        eprint
-                    _             -> mzero)
+               let baseUrl =
+                     case map toLower etype of
+                       "arxiv"       -> "http://arxiv.org/abs/"
+                       "jstor"       -> "http://www.jstor.org/stable/"
+                       "pubmed"      -> "http://www.ncbi.nlm.nih.gov/pubmed/"
+                       "googlebooks" -> "http://books.google.com?id="
+                       _             -> ""
+               if null baseUrl
+                  then mzero
+                  else return $ baseUrl ++ eprint)
        <|> return mempty
   doi' <- (guard (lookup "doi" opts /= Just "false") >> getRawField "doi")
          <|> return mempty
@@ -1509,6 +1542,7 @@ itemToReference lang locale bibtex caseTransform = bib $ do
 
   return $ emptyReference
          { refId               = Literal id'
+         , refOtherIds         = map Literal otherIds
          , refType             = reftype
          , author              = author'
          , editor              = editor'
