@@ -66,7 +66,7 @@ import           Data.Char           (isDigit, toLower)
 import           Data.Either         (lefts, rights)
 import           Data.Generics       hiding (Generic)
 import qualified Data.HashMap.Strict as H
-import           Data.List           (elemIndex)
+import           Data.List           (findIndex, elemIndex)
 import           Data.List.Split     (splitWhen)
 import           Data.Maybe          (fromMaybe, isNothing)
 import           Data.String
@@ -249,10 +249,16 @@ instance OVERLAPS
     circa' <- (v .: "circa" >>= parseBool) <|> pure False
     season' <- v .:? "season" >>= parseMaybeSeason
     case dateParts of
-         Just (Array xs) | isNothing raw' && not (V.null xs)
-                          -> mapM (fmap (setCirca circa' .
-                                   maybe id setSeason season') . parseJSON)
-                             $ V.toList xs
+         Just (Array y) | isNothing raw' ->
+           case V.toList y of
+              []           -> return []
+              [Null]       -> return []
+              [Array x]
+                | V.null x -> return []
+                   -- [ null ] and [ [] ] are sometimes seen. See
+                   -- https://github.com/greenelab/manubot/issues/66
+              ys           -> mapM (fmap (setCirca circa' .
+                                maybe id setSeason season') . parseJSON) ys
          _ -> case raw' of
                   Nothing -> handleLiteral <$> parseJSON (Object v)
                   Just r  -> return $ parseRawDate r
@@ -402,6 +408,7 @@ instance ToYaml CLabel where
 data Reference =
     Reference
     { refId                    :: Literal
+    , refOtherIds              :: [Literal]
     , refType                  :: RefType
 
     , author                   :: [Agent]
@@ -487,6 +494,7 @@ instance FromJSON Reference where
      v <- parseSuppFields v' <|> return v'
      addPageFirst <$> (Reference <$>
        v .:? "id" .!= "" <*>
+       v .:? "other-ids" .!= [] <*>
        v .:? "type" .!= NoType <*>
        v .:? "author" .!= [] <*>
        v .:? "editor" .!= [] <*>
@@ -614,6 +622,7 @@ regText = (T.pack <$> P.many1 (P.noneOf "\n{")) <|> (T.singleton <$> P.anyChar)
 instance ToJSON Reference where
   toJSON ref = object' [
       "id" .= refId ref
+    , "other-ids" .= refOtherIds ref
     , "type" .= refType ref
     , "author" .= author ref
     , "editor" .= editor ref
@@ -693,6 +702,7 @@ instance ToJSON Reference where
 instance ToYaml Reference where
   toYaml ref = mapping' [
       "id" &= refId ref
+    , "other-ids" &= refOtherIds ref
     , (("type" Y..= refType ref) :)
     , "author" &= author ref
     , "editor" &= editor ref
@@ -780,6 +790,7 @@ emptyReference :: Reference
 emptyReference =
     Reference
     { refId               = mempty
+    , refOtherIds         = mempty
     , refType             = NoType
 
     , author              = []
@@ -865,43 +876,81 @@ numericVars = [ "edition", "volume", "number-of-volumes", "number", "issue", "ci
               , "chapter-number", "collection-number", "number-of-pages"]
 
 getReference :: [Reference] -> Cite -> Maybe Reference
-getReference  r c
-    = case citeId c `elemIndex` map (unLiteral . refId) r of
-        Just i  -> Just $ setPageFirst $ r !! i
+getReference  rs c
+    = case (hasId (citeId c)) `findIndex` rs of
+        Just i  -> Just $ setPageFirst $ rs !! i
         Nothing -> Nothing
+  where hasId :: String -> Reference -> Bool
+        hasId ident r = ident `elem` (map unLiteral (refId r : refOtherIds r))
 
 processCites :: [Reference] -> [[Cite]] -> [[(Cite, Maybe Reference)]]
 processCites rs cs
-    = procGr [[]] cs
+    = procGr [] cs
     where
       procRef r = case filter ((==) (unLiteral $ refId r) . citeId) $ concat cs of
                     x:_ -> r { firstReferenceNoteNumber = readNum $ citeNoteNumber x}
                     []  -> r
 
       procGr _ [] = []
-      procGr a (x:xs) = let (a',res) = procCs a x
-                        in res : procGr (a' ++ [[]]) xs
+      procGr acc (x:xs) = let (a',res) = procCs acc x
+                          in res : procGr ([] : a') xs
 
-      procCs a [] = (a,[])
-      procCs a (c:xs)
-          | isIbid,  isLocSet = go "ibid-with-locator"
-          | isIbid            = go "ibid"
-          | isElem            = go "subsequent"
-          | otherwise         = go "first"
+      -- process, given the accumulated history, the current group's cites
+      procCs acc [] = (acc,[])
+      procCs acc (c:xs) = let (a, rest) = procCs addCite xs
+                              ref       = procRef <$> getReference rs c
+                              c'        = c { citePosition = getCitePosition }
+                          in  (a, (c', ref) : rest)
           where
-            go s = let addCite    = init a ++ [last a ++ [c]]
-                       (a', rest) = procCs addCite xs
-                   in  (a', (c { citePosition = s},
-                             procRef <$> getReference rs c) : rest)
-            isElem   = citeId c `elem` map citeId (concat a)
-            isIbid   = case reverse (last a) of
-                            []    -> case reverse (init a) of
-                                          []     -> False
-                                          (zs:_) -> not (null zs) &&
-                                                    all (== citeId c)
-                                                        (map citeId zs)
-                            (x:_) -> citeId c == citeId x
-            isLocSet = citeLocator c /= ""
+            addCite = case acc of
+                        []     -> [[c]]
+                        (a:as) -> (c : a) : as
+
+            -- http://docs.citationstyles.org/en/stable/specification.html#locators
+            getCitePosition = fromMaybe notIbid (ibidPosition <$> prevSameCite)
+                where
+                    notIbid = if citeId c `elem` map citeId (concat acc)
+                                 then "subsequent"
+                                 else "first"
+
+            ibidPosition x = let hasL k   = citeLocator k /= ""
+                                 withIf b = if b then "ibid-with-locator" else "ibid"
+                                 diffLoc  = citeLocator x /= citeLocator c
+                                           || citeLabel x /= citeLabel c
+                             in  case (hasL x, hasL c) of
+                                   (False, cur)  -> withIf cur
+                                   (True, True)  -> withIf diffLoc
+                                   (True, False) -> "subsequent"
+
+            -- x is previous cite in current group
+            -- zs is the previous group
+            prevSameCite = case acc of
+                             []     -> Nothing
+                             (a:as) -> psc a as
+              where
+                -- you can't have an ibid at the start of your document
+                psc [] []     = Nothing
+
+                -- a. the current cite immediately follows on another cite,
+                --    within the same citation, that references the same item
+                psc (x:_) _   = if citeId c == citeId x
+                                   then Just x
+                                   else Nothing
+
+                -- b.  [] => the current cite is the first cite in the citation
+                --     zs => and the previous citation consists of a single cite
+                --           that refs the same item
+                -- The spec appears to be concerned that you cannot know the
+                -- correct one to match the locator against.
+                -- It is a super clunky if you have [@a, 1; @a, 2] then [@a, 3],
+                -- where the second citation gets "subsequent" even though there were
+                -- only @a keys.
+                -- This is silly. We will use the last one to match against.
+                psc [] (zs:_) = case zs of
+                                  [] -> Nothing
+                                  (z:_) -> if all (== citeId c) (map citeId zs)
+                                              then Just z
+                                              else Nothing
 
 setPageFirst :: Reference -> Reference
 setPageFirst ref =
@@ -1033,24 +1082,24 @@ rawDateOld = do
   let sep = P.oneOf [' ','/',','] >> P.spaces
   let rangesep = P.try $ P.spaces >> P.char '-' >> P.spaces
   let refDate = RefDate Nothing Nothing Nothing Nothing mempty False
-  let date = P.choice $ map P.try [
-                 (do s <- pseason
+  let date = P.choice $ map P.try
+                [ do s <- pseason
                      sep
                      y <- pyear
-                     return refDate{ year = y, season = s })
-               , (do m <- pmonth
+                     return refDate{ year = y, season = s }
+                , do m <- pmonth
                      sep
                      d <- pday
                      sep
                      y <- pyear
-                     return refDate{ year = y, month = m, day = d })
-               , (do m <- pmonth
+                     return refDate{ year = y, month = m, day = d }
+                , do m <- pmonth
                      sep
                      y <- pyear
-                     return refDate{ year = y, month = m })
-               , (do y <- pyear
-                     return refDate{ year = y })
-               ]
+                     return refDate{ year = y, month = m }
+                , do y <- pyear
+                     return refDate{ year = y }
+                ]
   d1 <- date
   P.option [d1] ((\x -> [d1,x]) <$> (rangesep >> date))
 
